]> git.cworth.org Git - empires-server/blobdiff - game.js
Rename "next_player" property to "team_to_play"
[empires-server] / game.js
diff --git a/game.js b/game.js
index f1f764fa1968425d597e4e2ab47e4b8d916650a8..8a0e0834c0a48587e24e6ff3d4589224b87a2a3b 100644 (file)
--- a/game.js
+++ b/game.js
@@ -1,9 +1,53 @@
+/* A single player can have multiple connections, (think, multiple
+ * browser windows with a common session cookie). */
+class Player {
+  constructor(id, name, connection) {
+    this.id = id;
+    this.name = name;
+    this.connections = [connection];
+    this.team = "";
+  }
+
+  add_connection(connection) {
+    /* Don't add a duplicate connection if this player already has it. */
+    for (let c of this.connections) {
+      if (c === connection)
+        return;
+    }
+
+    this.connections.push(connection);
+  }
+
+  /* Returns the number of remaining connections after this one is removed. */
+  remove_connection(connection) {
+    this.connections.filter(c => c !== connection);
+    return this.connections.length;
+  }
+
+  /* Send a string to all connections for this player. */
+  send(data) {
+    this.connections.forEach(connection => connection.write(data));
+  }
+
+  info_json() {
+    return JSON.stringify({
+      id: this.id,
+      name: this.name,
+      team: this.team
+    });
+  }
+}
+
 /* Base class providing common code for game engine implementations. */
 class Game {
   constructor(id) {
     this.id = id;
-    this.clients = [];
-    this.next_client_id = 1;
+    this.players = [];
+    this.next_player_id = 1;
+    this.teams = [];
+
+    /* Send a comment to every connected client every 15 seconds. */
+    setInterval(() => {this.broadcast_string(":");}, 15000);
   }
 
   /* Suport for game meta-data.
@@ -26,25 +70,47 @@ class Game {
     return this._meta;
   }
 
-  add_client(response) {
-    const id = this.next_client_id;
-    this.clients.push({id: id,
-                       response: response});
-    this.next_client_id++;
+  add_player(session, connection) {
+    /* First see if we already have a player object for this session. */
+    const existing = this.players[session.id];
+    if (existing) {
+      existing.add_connection(connection);
+      return existing;
+    }
+
+    /* No existing player. Add a new one. */
+    const id = this.next_player_id;
+    const player = new Player(id, session.nickname, connection);
 
-    return id;
+    /* Broadcast before adding player to list (to avoid announcing the
+     * new player to itself). */
+    const player_data = JSON.stringify({ id: player.id, name: player.name });
+    this.broadcast_event("player-enter", player_data);
+
+    this.players[session.id] = player;
+    this.next_player_id++;
+
+    return player;
   }
 
-  remove_client(id) {
-    this.clients = this.clients.filter(client => client.id !== id);
+  /* Drop a connection object from a player, and if it's the last one,
+   * then drop that player from the game's list of players. */
+  remove_player_connection(player, connection) {
+    const remaining = player.remove_connection(connection);
+    if (remaining === 0) {
+      const player_data = JSON.stringify({ id: player.id });
+      this.players.filter(p => p !== player);
+      this.broadcast_event("player-exit", player_data);
+    }
   }
 
/* Send a string to all clients */
   broadcast_string(str) {
-    this.clients.forEach(client => client.response.write(str + '\n'));
+    for (let [session_id, player] of Object.entries(this.players))
+      player.send(str + '\n');
   }
 
/* Send an event to all clients.
    *
    * An event has both a declared type and a separate data block.
    * It also ends with two newlines (to mark the end of the event).
@@ -62,13 +128,23 @@ class Game {
     };
     response.writeHead(200, headers);
 
-    /* Add this new client to our list of clients. */
-    const id = this.add_client(response);
+    /* Add this new player. */
+    const player = this.add_player(request.session, response);
 
     /* And queue up cleanup to be triggered on client close. */
     request.on('close', () => {
-      this.remove_client(id);
+      this.remove_player_connection(player, response);
+    });
+
+    /* Give the client the game-info event. */
+    const game_info_json = JSON.stringify({
+      id: this.id,
+      url: `${request.protocol}://${request.hostname}/${this.id}`
     });
+    response.write(`event: game-info\ndata: ${game_info_json}\n\n`);
+
+    /* And the player-info event. */
+    response.write(`event: player-info\ndata: ${player.info_json()}\n\n`);
 
     /* Finally, if this game class has a "state" property, stream that
      * current state to the client. */
@@ -78,6 +154,41 @@ class Game {
     }
   }
 
+  handle_player(request, response) {
+    const player = this.players[request.session.id];
+    const name = request.body.name;
+    const team = request.body.team;
+    var updated = false;
+    if (! player) {
+      response.sendStatus(404);
+      return;
+    }
+
+    if (name && (player.name !== name)) {
+      player.name = name;
+
+      /* In addition to setting the name within this game's player
+       * object, also set the name in the session. */
+      request.session.nickname = name;
+      request.session.save();
+
+      updated = true;
+    }
+
+    if (team !== null && (player.team !== team) &&
+        (team === "" || this.teams.includes(team)))
+    {
+      player.team = team;
+
+      updated = true;
+    }
+
+    if (updated)
+      this.broadcast_event("player-update", player.info_json());
+
+    response.send("");
+  }
+
   broadcast_move(move) {
     this.broadcast_event("move", move);
   }
@@ -26,25 +70,47 @@ class Game {
   }
 
-  /* Send a string to all clients */
+  /* Send a string to all players */
   broadcast_string(str) {
@@ -26,25 +70,47 @@ class Game {
   }
 
-  /* Send an event to all clients.
+  /* Send an event to all players.
    *
    * An event has both a declared type and a separate data block.